"use strict";
var __awaiter =
  (this && this.__awaiter) ||
  function (thisArg, _arguments, P, generator) {
    function adopt(value) {
      return value instanceof P
        ? value
        : new P(function (resolve) {
            resolve(value);
          });
    }
    return new (P || (P = Promise))(function (resolve, reject) {
      function fulfilled(value) {
        try {
          step(generator.next(value));
        } catch (e) {
          reject(e);
        }
      }
      function rejected(value) {
        try {
          step(generator["throw"](value));
        } catch (e) {
          reject(e);
        }
      }
      function step(result) {
        result.done
          ? resolve(result.value)
          : adopt(result.value).then(fulfilled, rejected);
      }
      step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
  };
Object.defineProperty(exports, "__esModule", { value: true });
exports.getPreferredLanguage = void 0;
const geoip_lite_1 = require("geoip-lite");
// Default language to use if no other language is found
const DEFAULT_LANGUAGE = "en";
// This function gets the language from local storage, if it exists
function getLanguageFromStorage() {
  // Check if we're running in a browser
  if (typeof window === "undefined") {
    return null;
  }
  // Get the language from local storage
  const storageLanguage = localStorage.getItem("language");
  // Check if the language code is valid
  if (storageLanguage === "vi" || storageLanguage === "en") {
    return storageLanguage;
  }
  // Return null if the language code is invalid or doesn't exist
  return null;
}
// This function gets the language from the user's IP address, if possible
function getLanguageFromIP(ip) {
  var _a;
  // Use geoip-lite to get the user's country code from their IP address
  const countryCode =
    (_a = (0, geoip_lite_1.lookup)(ip)) === null || _a === void 0
      ? void 0
      : _a.country;
  // Map the country code to a language code
  if (countryCode === "VN") {
    return "vi";
  }
  // Return the default language if the country code is unknown or not supported
  return DEFAULT_LANGUAGE;
}
// This function gets the user's IP address from the browser
function getUserIP() {
  return __awaiter(this, void 0, void 0, function* () {
    // Check if we're running in a browser
    if (typeof window === "undefined") {
      return null;
    }
    try {
      // Use a free API to get the user's IP address
      const response = yield fetch("https://api.ipify.org?format=json");
      const data = yield response.json();
      return data.ip;
    } catch (error) {
      console.error(error);
      return null;
    }
  });
}
// This function sets the language code in local storage
function setLanguageToStorage(languageCode) {
  // Check if we're running in a browser
  if (typeof window === "undefined") {
    return;
  }
  // Set the language in local storage
  localStorage.setItem("language", languageCode);
}
// This function gets the user's preferred language, using local storage if available, or falling back to the user's IP address
function getPreferredLanguage() {
  return __awaiter(this, void 0, void 0, function* () {
    // Try to get the language from local storage
    const languageFromStorage = getLanguageFromStorage();
    if (languageFromStorage) {
      return languageFromStorage;
    }
    // Try to get the language from the user's IP address
    const userIP = yield getUserIP();
    if (userIP) {
      const languageFromIP = getLanguageFromIP(userIP);
      if (languageFromIP) {
        // Set the language in local storage for future use
        setLanguageToStorage(languageFromIP);
        return languageFromIP;
      }
    }
    // Return the default language if all else fails
    return DEFAULT_LANGUAGE;
  });
}
exports.getPreferredLanguage = getPreferredLanguage;
