# Language Detection Code

This TypeScript code uses [geoip-lite](https://www.npmjs.com/package/geoip-lite) to detect a user's country code from their IP address and map it to a valid language code ("vi" for Vietnamese or "en" for English). It also uses local storage to remember the user's preferred language for future visits.

## Usage

To use this code in your TypeScript project:

1. Install [geoip-lite](https://www.npmjs.com/package/geoip-lite) by running `npm install geoip-lite`.
2. Copy the code into your project, making sure to import the `getPreferredLanguage` function.
3. Call `getPreferredLanguage` to get the user's preferred language code.

## Code Explanation

### Type Definition

The `LanguageCode` type defines the two valid language codes: "vi" for Vietnamese and "en" for English.

### Constants

The `DEFAULT_LANGUAGE` constant specifies the language code to use if no other language is found.

### Functions

- `getLanguageFromStorage()`: This function gets the language from local storage, if it exists. It checks if the language code is valid and returns it, or returns `null` if the language code is invalid or doesn't exist.

- `getLanguageFromIP(ip: string)`: This function gets the language from the user's IP address, using `geoip-lite` to get the user's country code from their IP address. It maps the country code to a language code ("vi" for Vietnam or "en" for other countries). If the country code is unknown or not supported, it returns the default language.

- `getUserIP()`: This function gets the user's IP address from the browser, using a free API. It returns the IP address or `null` if an error occurs.

- `setLanguageToStorage(languageCode: LanguageCode)`: This function sets the language code in local storage.

- `getPreferredLanguage()`: This function gets the user's preferred language, using local storage if available, or falling back to the user's IP address. It first tries to get the language from local storage. If that fails, it tries to get the language from the user's IP address and sets it in local storage for future use. If that also fails, it returns the default language.

## Example

```typescript
import { getPreferredLanguage } from "./languageDetection";

async function example() {
  const language = await getPreferredLanguage();
  console.log(language);
}

example();
```

This code logs the user's preferred language to the console.
